"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRegisterForeignContractIx = exports.createUpdateRelayerFeeIx = exports.createInitializeIx = exports.getCompleteTransferWrappedWithPayloadCpiAccounts = exports.createReceiveRndrV1Ix = void 0;
const web3_js_1 = require("@solana/web3.js");
const solana_1 = require("@certusone/wormhole-sdk/lib/cjs/solana");
const wormhole_1 = require("@certusone/wormhole-sdk/lib/cjs/solana/wormhole");
const spl_token_1 = require("@solana/spl-token");
const wormhole_sdk_1 = require("@certusone/wormhole-sdk");
const tokenBridge_1 = require("@certusone/wormhole-sdk/lib/cjs/solana/tokenBridge");
const pdas_1 = require("./pdas");
const bn_js_1 = __importDefault(require("bn.js"));
const circuit_breaker_sdk_1 = require("@render-foundation/circuit-breaker-sdk");
const spl_utils_1 = require("@render-foundation/spl-utils");
function createReceiveRndrV1Ix(program, payer, tokenBridgeProgramId, wormholeProgramId, wormholeMessage, accts) {
    const parsed = (0, wormhole_sdk_1.isBytes)(wormholeMessage)
        ? (0, wormhole_sdk_1.parseTokenTransferVaa)(wormholeMessage)
        : wormholeMessage;
    const wrappedMint = (0, tokenBridge_1.deriveWrappedMintKey)(tokenBridgeProgramId, parsed.tokenChain, parsed.tokenAddress);
    const tokenBridgeAccounts = getCompleteTransferWrappedWithPayloadCpiAccounts(tokenBridgeProgramId, wormholeProgramId, payer, parsed);
    const recipient = new web3_js_1.PublicKey(parsed.tokenTransferPayload.subarray(1, 33));
    /*const recipientTokenAccount = getAssociatedTokenAddressSync(
      accts.rndrMint,
      recipient
    )*/
    return program.methods
        .receiveRenderV1({ vaaHash: [...parsed.hash] })
        .accounts(Object.assign({ config: (0, pdas_1.redeemerConfigKey)()[0], foreignContract: (0, pdas_1.foreignContractKey)(parsed.emitterChain)[0], recipient, transfer: (0, pdas_1.transferKey)(parsed.hash)[0], rndrMint: accts.rndrMint, circuitBreaker: accts.circuitBreakerK, bridge: accts.bridgeK, payerTokenAccount: (0, spl_token_1.getAssociatedTokenAddressSync)(wrappedMint, new web3_js_1.PublicKey(payer)), tokenBridgeProgram: new web3_js_1.PublicKey(tokenBridgeProgramId), wrndrMint: tokenBridgeAccounts.tokenBridgeWrappedMint }, tokenBridgeAccounts))
        .instruction();
}
exports.createReceiveRndrV1Ix = createReceiveRndrV1Ix;
// Temporary
function getCompleteTransferWrappedWithPayloadCpiAccounts(tokenBridgeProgramId, wormholeProgramId, payer, vaa) {
    const parsed = (0, wormhole_sdk_1.isBytes)(vaa) ? (0, wormhole_sdk_1.parseTokenTransferVaa)(vaa) : vaa;
    const mint = (0, tokenBridge_1.deriveWrappedMintKey)(tokenBridgeProgramId, parsed.tokenChain, parsed.tokenAddress);
    const cpiProgramId = new web3_js_1.PublicKey(parsed.to);
    return {
        payer: new web3_js_1.PublicKey(payer),
        tokenBridgeConfig: (0, tokenBridge_1.deriveTokenBridgeConfigKey)(tokenBridgeProgramId),
        vaa: (0, wormhole_1.derivePostedVaaKey)(wormholeProgramId, parsed.hash),
        tokenBridgeClaim: (0, wormhole_1.deriveClaimKey)(tokenBridgeProgramId, parsed.emitterAddress, parsed.emitterChain, parsed.sequence),
        tokenBridgeForeignEndpoint: (0, tokenBridge_1.deriveEndpointKey)(tokenBridgeProgramId, parsed.emitterChain, parsed.emitterAddress),
        tokenBridgeRedeemer: (0, tokenBridge_1.deriveRedeemerAccountKey)(cpiProgramId),
        tokenBridgeWrappedMint: mint,
        tokenBridgeWrappedMeta: (0, tokenBridge_1.deriveWrappedMetaKey)(tokenBridgeProgramId, mint),
        tokenBridgeMintAuthority: (0, tokenBridge_1.deriveMintAuthorityKey)(tokenBridgeProgramId),
        rent: web3_js_1.SYSVAR_RENT_PUBKEY,
        systemProgram: web3_js_1.SystemProgram.programId,
        tokenProgram: spl_token_1.TOKEN_PROGRAM_ID,
        wormholeProgram: new web3_js_1.PublicKey(wormholeProgramId),
    };
}
exports.getCompleteTransferWrappedWithPayloadCpiAccounts = getCompleteTransferWrappedWithPayloadCpiAccounts;
function createInitializeIx(program, tokenBridgeProgramId, wormholeProgramId, accts, args = { relayerFee: 2, relayerFeePrecision: 3 }) {
    const { owner, wRndrMint, rndrMint, ethTokenAddress, ethTokenChain, bridgeK, circuitBreakerK } = accts;
    const { relayerFee, relayerFeePrecision } = args;
    const tokenBridgeAccounts = (0, solana_1.getTokenBridgeDerivedAccounts)(program.programId, tokenBridgeProgramId, wormholeProgramId);
    return program.methods
        // @ts-ignore
        .initializeBridgeV1({
        relayerFee,
        relayerFeePrecision,
        ethTokenAddress: ethTokenAddress,
        ethTokenChain: ethTokenChain,
        config: {
            windowSizeSeconds: new bn_js_1.default(60 * 60),
            thresholdType: circuit_breaker_sdk_1.ThresholdType.Absolute,
            threshold: (0, spl_utils_1.toBN)(10, rndrMint.decimals)
        },
        incentiveThreshold: new bn_js_1.default(1000)
    })
        .accounts(Object.assign({ owner: owner, bridge: (0, pdas_1.bridgeKey)(rndrMint.address, wRndrMint)[0], rndrMint: rndrMint.address, wrndrMint: wRndrMint, circuitBreaker: circuitBreakerK, redeemerConfig: (0, pdas_1.redeemerConfigKey)(program.programId)[0], tokenBridgeProgram: new web3_js_1.PublicKey(tokenBridgeProgramId), wormholeProgram: new web3_js_1.PublicKey(wormholeProgramId) }, tokenBridgeAccounts)).instruction();
}
exports.createInitializeIx = createInitializeIx;
function createUpdateRelayerFeeIx(program, payer, relayerFee, relayerFeePrecision) {
    return __awaiter(this, void 0, void 0, function* () {
        return program.methods
            .updateRelayerFeeV1({ relayerFee, relayerFeePrecision })
            .accounts({
            owner: new web3_js_1.PublicKey(payer),
            config: (0, pdas_1.redeemerConfigKey)()[0],
        })
            .instruction();
    });
}
exports.createUpdateRelayerFeeIx = createUpdateRelayerFeeIx;
function createRegisterForeignContractIx(program, payer, tokenBridgeProgramId, chain, contractAddress, tokenBridgeForeignAddress) {
    return __awaiter(this, void 0, void 0, function* () {
        return program.methods
            .registerForeignContractV1({ chain, address: [...contractAddress] })
            .accounts({
            owner: new web3_js_1.PublicKey(payer),
            foreignContract: (0, pdas_1.foreignContractKey)(chain)[0],
            tokenBridgeForeignEndpoint: (0, tokenBridge_1.deriveEndpointKey)(tokenBridgeProgramId, chain, tokenBridgeForeignAddress),
            tokenBridgeProgram: new web3_js_1.PublicKey(tokenBridgeProgramId),
        })
            .instruction();
    });
}
exports.createRegisterForeignContractIx = createRegisterForeignContractIx;
//# sourceMappingURL=instructions.js.map